<?php

declare(strict_types=1);

namespace Dedi\SyliusBannerPlugin\Entity;

use DateTime;
use DateTimeImmutable;
use DateTimeInterface;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Mapping\Annotation as Gedmo;
use Sylius\Component\Resource\Model\TimestampableTrait;
use Sylius\Component\Resource\Model\TranslatableTrait;
use Sylius\Component\Resource\Model\TranslationInterface;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * @ORM\Entity
 * @ORM\Table(name="dedi_banner_alert_message")
 */
class AlertMessage implements AlertMessageInterface
{
    use TimestampableTrait;
    use TranslatableTrait {
        __construct as private initializeTranslationsCollection;
    }

    /**
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Column(type="integer", nullable=false, name="id")
     */
    private ?int $id;

    /** @ORM\Column(name="enabled", type="boolean", nullable=false) */
    private bool $enabled = false;

    /** @ORM\Column(name="start_date_time", type="datetime_immutable", nullable=true) */
    private ?DateTimeImmutable $startDateTime;

    /**
     * @ORM\Column(name="end_date_time", type="datetime_immutable", nullable=true)
     * @Assert\Expression(
     *     expression="value == null || value >= this.getStartDateTime()",
     *     groups={"default", "sylius"},
     *     message="dedi_banner.alert_message.end_date_can_t_be_before_start_date"
     * )
     */
    private ?DateTimeImmutable $endDateTime;

    /** @ORM\Column(name="type", type="string", nullable=false) */
    private string $type = AlertMessageEnum::TOP_BAR;

    /**
     * @var DateTimeInterface
     *
     * @ORM\Column(type="datetime", name="created_at", nullable=true)
     * @Gedmo\Timestampable(on="create")
     */
    protected $createdAt;

    /**
     * @var DateTimeInterface
     *
     * @ORM\Column(type="datetime", name="update_at", nullable=true)
     * @Gedmo\Timestampable(on="update")
     */
    protected $updatedAt;

    /** @ORM\Column(type="string", name="color", nullable=true) */
    private ?string $color = null;

    public function __construct()
    {
        $this->initializeTranslationsCollection();
    }

    protected function createTranslation(): TranslationInterface
    {
        return new AlertMessageTranslation();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function isEnabled(): bool
    {
        return $this->enabled;
    }

    public function setEnabled(bool $enabled): AlertMessageInterface
    {
        $this->enabled = $enabled;

        return $this;
    }

    public function getStartDateTime(): ?DateTimeImmutable
    {
        return $this->startDateTime;
    }

    public function setStartDateTime(?DateTimeImmutable $startDateTime): AlertMessageInterface
    {
        $this->startDateTime = $startDateTime;

        return $this;
    }

    public function getEndDateTime(): ?DateTimeImmutable
    {
        return $this->endDateTime;
    }

    public function setEndDateTime(?DateTimeImmutable $endDateTime): AlertMessageInterface
    {
        $this->endDateTime = $endDateTime;

        return $this;
    }

    public function getType(): ?string
    {
        return $this->type;
    }

    public function setType(?string $type): self
    {
        $this->type = $type;

        return $this;
    }

    public function getMessage(): ?string
    {
        /** @var AlertMessageTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getMessage();
    }

    public function getLink(): ?string
    {
        /** @var AlertMessageTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getLink();
    }

    public function getLinkLabel(): ?string
    {
        /** @var AlertMessageTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getLinkLabel();
    }

    public function getTitle(): ?string
    {
        /** @var AlertMessageTranslationInterface $translation */
        $translation = $this->getTranslation();

        return $translation->getTitle();
    }

    public function getColor(): ?string
    {
        return $this->color;
    }

    public function setColor(?string $color): self
    {
        $this->color = $color;

        return $this;
    }
}
